@@ -1,7 +1,8 @@
 /*
- * Copyright (c) 2019 Yubico AB. All rights reserved.
+ * Copyright (c) 2019-2022 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <assert.h>
@@ -19,6 +20,7 @@ int fido_nfc_rx(fido_dev_t *, uint8_t, unsigned char *, size_t, int);
 int fido_nfc_tx(fido_dev_t *, uint8_t, const unsigned char *, size_t);
 size_t LLVMFuzzerMutate(uint8_t *, size_t, size_t);
 
+extern int prng_up;
 static const uint8_t *wire_data_ptr = NULL;
 static size_t wire_data_len = 0;
 
@@ -133,12 +135,18 @@ void
 mutate_byte(uint8_t *b)
 {
 	LLVMFuzzerMutate(b, sizeof(*b), sizeof(*b));
+#ifdef WITH_MSAN
+	__msan_unpoison(b, sizeof(*b));
+#endif
 }
 
 void
 mutate_int(int *i)
 {
 	LLVMFuzzerMutate((uint8_t *)i, sizeof(*i), sizeof(*i));
+#ifdef WITH_MSAN
+	__msan_unpoison(i, sizeof(*i));
+#endif
 }
 
 void
@@ -157,7 +165,6 @@ mutate_string(char *s)
 	s[n] = '\0';
 }
 
-/* XXX should fail, but doesn't */
 static int
 buf_read(unsigned char *ptr, size_t len, int ms)
 {
@@ -165,6 +172,11 @@ buf_read(unsigned char *ptr, size_t len, int ms)
 
 	(void)ms;
 
+	if (prng_up && uniform_random(400) < 1) {
+		errno = EIO;
+		return -1;
+	}
+
 	if (wire_data_len < len)
 		n = wire_data_len;
 	else
@@ -183,7 +195,7 @@ buf_write(const unsigned char *ptr, size_t len)
 {
 	consume(ptr, len);
 
-	if (uniform_random(400) < 1) {
+	if (prng_up && uniform_random(400) < 1) {
 		errno = EIO;
 		return -1;
 	}
